package dev.fyloz.trial.colorrecipesexplorer;

import dev.fyloz.trial.colorrecipesexplorer.core.io.file.FileHandler;
import dev.fyloz.trial.colorrecipesexplorer.core.services.PasswordService;
import dev.fyloz.trial.colorrecipesexplorer.core.services.files.FilesService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.context.MessageSource;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.util.List;

@SpringBootApplication
public class ColorRecipesExplorerApplication {

    public static final Logger LOGGER = LoggerFactory.getLogger(ColorRecipesExplorerApplication.class);
    public static String UPLOAD_LOCATION;
    public static final String USERS_FILE_NAME = "passwords";
    public static boolean USE_PORT;

    public static ColorRecipesExplorerApplication CREApp;

    private MessageSource messageSource;
    private FilesService filesService;

    public static void main(String[] args) {
        UPLOAD_LOCATION = args[0] != null ? args[0] : "./";

        SpringApplication.run(ColorRecipesExplorerApplication.class, args);
    }

    @Autowired
    public ColorRecipesExplorerApplication(MessageSource messageSource, FilesService filesService) {
        this.messageSource = messageSource;
        this.filesService = filesService;

        CREApp = this;
        LOGGER.info("Le fichier des utilisateurs se situe à: " + new File(UPLOAD_LOCATION + "/" + USERS_FILE_NAME).getAbsolutePath());

        loadPasswords();
    }

    /**
     * Charge les mots de passes contenus dans le fichier.
     * <p>
     * Un mot de passe correspond à une ligne dans le fichier passwords.txt.
     */
    private void loadPasswords() {
        String filePath = String.format("%s/%s.txt", UPLOAD_LOCATION, USERS_FILE_NAME);

        try {
            if(filesService.fileExists(filePath)) filesService.createFile(filePath);
            List<String> fileContent = filesService.readFileAsStrings(filePath);

            if (fileContent.size() < 1) {
                LOGGER.warn("Aucun mot de passe trouvé. Il sera impossible d'utiliser certaines fonctionnalités de l'application.");
            }

            for (String line : fileContent) {
                PasswordService.addPassword(line);
            }
        } catch (IOException e) {
            LOGGER.error("Une erreur est survenue lors du chargement du fichier des utilisateurs", e);
            LOGGER.warn("Il sera impossible d'utiliser certaines fonctionnalités de l'application.");
        }
    }

    public MessageSource getMessageSource() {
        return messageSource;
    }
}
