package dev.fyloz.trial.colorrecipesexplorer.core.model;

import com.fasterxml.jackson.annotation.JsonIgnore;
import lombok.*;
import org.hibernate.validator.constraints.Length;

import javax.persistence.*;
import javax.validation.constraints.NotNull;
import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;

@Entity
@Data
@EqualsAndHashCode(callSuper = false)
@RequiredArgsConstructor
@NoArgsConstructor
public class Recipe implements IModel {

    @Id
    @GeneratedValue(strategy = GenerationType.SEQUENCE)
    private Long id;

    @NonNull
    @NotNull
    @Length(min = 2)
    private String name;

    @NonNull
    @NotNull
    @ManyToOne
    private Company company;

    @NonNull
    @NotNull
    private String description;

    @NonNull
    @NotNull
    private Integer sample;

    private String approbationDate;

    private String remark;

    private String note;

    @JsonIgnore
    @OneToMany(mappedBy = "recipe", cascade = CascadeType.ALL)
    private List<Mix> mixes;

    @OneToMany(mappedBy = "recipe", cascade = CascadeType.ALL)
    private List<RecipeStep> recipeSteps;

    public Collection<MixType> getMixTypes() {
        return mixes.stream()
                .map(Mix::getMixType)
                .collect(Collectors.toList());
    }

    public boolean hasMixType(MixType mixType) {
        return getMixTypes().contains(mixType);
    }

}
