package dev.fyloz.trial.colorrecipesexplorer.core.services.files;

import dev.fyloz.trial.colorrecipesexplorer.ColorRecipesExplorerApplication;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.imageio.ImageIO;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;

@Service
public class ImagesService {

    private static final String IMAGES_DIRECTORY = "images";

    private FilesService filesService;

    @Autowired
    public ImagesService(FilesService filesService) {
        this.filesService = filesService;
    }

    public byte[] readImage(String name) {
        try {
            return filesService.readFileAsBytes(getPath(name));
        } catch (IOException ex) {
            throw new RuntimeException("Erreur lors de la lecture d'une image: " + ex.getMessage());
        }
    }

    public boolean writeMultipartImage(MultipartFile image, String name) {
        return filesService.writeMultiPartFile(image, getPath(name));
    }

    public void deleteImage(String name) {
        filesService.deleteFile(getPath(name));
    }

    public boolean isImage(InputStream input) {
        try {
            return !(ImageIO.read(input) == null);
        } catch (IOException ex) {
            throw new RuntimeException("Erreur lors de la vérification d'une image: " + ex.getMessage());
        }
    }

    public File getDirectoryFile() {
        return new File(getPath(""));
    }

    private String getPath(String name) {
        return String.format("%s/%s/%s", ColorRecipesExplorerApplication.UPLOAD_LOCATION, IMAGES_DIRECTORY, name);
    }

}
