package dev.fyloz.trial.colorrecipesexplorer.core.services.files;

import dev.fyloz.trial.colorrecipesexplorer.ColorRecipesExplorerApplication;
import dev.fyloz.trial.colorrecipesexplorer.core.model.Material;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.MaterialService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;

@Service
public class SimdutService {

    private static final String SIMDUT_DIRECTORY = "simdut";

    private FilesService filesService;
    private MaterialService materialService;

    @Autowired
    public SimdutService(FilesService filesService, MaterialService materialService) {
        this.filesService = filesService;
        this.materialService = materialService;
    }

    /**
     * Lit le fichier SIMDUT d'un produit et retourne son contenu.
     *
     * @param material Le produit
     * @return Le contenu du fichier SIMDUT du produit
     */
    public byte[] readSimdutForMaterial(Material material) {
        String path = getPath(material);

        if (filesService.fileExists(path)) return new byte[0];

        try {
            return filesService.readFileAsBytes(path);
        } catch (IOException ex) {
            throw new RuntimeException("Impossible de lire un fichier SIMDUT: " + ex.getMessage());
        }
    }

    /**
     * Lit le fichier SIMDUT du produit correspondant à un identifiant et retourne son contenu.
     *
     * @param id L'identifiant du produit
     * @return Le contenu du fichier SIMDUT du produit correspondant à l'identifiant
     */
    public byte[] readSimdutForMaterialId(Long id) {
        return readSimdutForMaterial(materialService.getById(id));
    }

    /**
     * Vérifie si un produit a un fichier SIMDUT.
     *
     * @param material Le produit
     * @return Si le produit a un fichier SIMDUT
     */
    public boolean simdutExistsForMaterial(Material material) {
        return filesService.fileExists(getPath(material));
    }

    /**
     * Vérifie si le produit correspondant à un identifiant a un fichier SIMDUT.
     *
     * @param id L'identifiant du produit
     * @return si le produit correspondant à l'identifiant a un fichier SIMDUT
     */
    public boolean simdutExistsForMaterialId(Long id) {
        return simdutExistsForMaterial(materialService.getById(id));
    }

    private String getPath(Material material) {
        return String.format("%s/%s/%s", ColorRecipesExplorerApplication.UPLOAD_LOCATION, SIMDUT_DIRECTORY, materialService.getSimdutFileName(material));
    }

}
