package dev.fyloz.trial.colorrecipesexplorer.core.services.model;

import dev.fyloz.trial.colorrecipesexplorer.core.exception.model.EntityAlreadyExistsException;
import dev.fyloz.trial.colorrecipesexplorer.core.exception.model.EntityNotFoundException;
import dev.fyloz.trial.colorrecipesexplorer.core.exception.model.ModelException;
import dev.fyloz.trial.colorrecipesexplorer.core.model.MaterialType;
import dev.fyloz.trial.colorrecipesexplorer.core.model.dto.MaterialTypeEditorDto;
import dev.fyloz.trial.colorrecipesexplorer.core.services.GenericService;
import dev.fyloz.trial.colorrecipesexplorer.dao.MaterialTypeDao;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class MaterialTypeService extends GenericService<MaterialType, MaterialTypeDao> {

    private MaterialService materialService;

    @Autowired
    public MaterialTypeService(MaterialTypeDao materialTypeDao, MaterialService materialService) {
        super(materialTypeDao, MaterialType.class);
        this.materialService = materialService;
    }

    public boolean isLinkedToMaterials(MaterialType materialType) {
        return !materialService.getAllByMaterialType(materialType).isEmpty();
    }

    @Override
    public MaterialType update(MaterialType entity) {
        throw new UnsupportedOperationException("Cette méthode n'est pas supportée pour les types de produits. Utiliser MaterialTypeService::update(MaterialTypeEditorDto)");
    }

    public MaterialType update(MaterialTypeEditorDto materialTypeDto) {
        MaterialType materialType = materialTypeDto.getMaterialType();

        if (!existsByName(materialTypeDto.getOldName()))
            throw new EntityNotFoundException(type, ModelException.IdentifierType.NAME, materialTypeDto.getOldName());
        if (materialTypeDto.getOldName().equals(materialType.getName()) && existsByName(materialType.getName()))
            throw new EntityAlreadyExistsException(type, ModelException.IdentifierType.NAME, materialType.getName());
        if (existsByPrefix(materialType.getPrefix()) && !getByPrefix(materialType.getPrefix()).getId().equals(materialType.getId()))
            throw new EntityAlreadyExistsException(type, ModelException.IdentifierType.OTHER, MaterialType.IDENTIFIER_PREFIX_NAME, materialType.getPrefix());

        return dao.save(materialType);
    }

    @Deprecated(since = "1.3.0", forRemoval = true)
    public void deleteIfNotLinked(MaterialType materialType) {
        if (!isLinkedToMaterials(materialType)) {
            delete(materialType);
        }
    }

    @Deprecated(since = "1.3.0", forRemoval = true)
    public boolean isValidForUpdateName(MaterialType materialType) {
        MaterialType materialTypeByName = dao.findByName(materialType.getName());

        return materialType.getId().equals(materialTypeByName.getId());
    }

    @Deprecated(since = "1.3.0", forRemoval = true)
    public boolean isValidForUpdatePrefix(MaterialType materialType) {
        MaterialType materialTypeByPrefix = dao.findByPrefix(materialType.getPrefix());

        return materialType.getId().equals(materialTypeByPrefix.getId());
    }

    /**
     * Récupère un type de produit par son nom.
     *
     * @param name Le nom du type de produit à récupérer
     * @return Le type de produit correspondant au nom.
     */
    public MaterialType getByName(String name) {
        return dao.findByName(name);
    }

    /**
     * Récupère un type de produit par son préfixe.
     *
     * @param prefix Le préfixe du type de produit à récupérer
     * @return Le type de produit correspondant au préfixe
     */
    public MaterialType getByPrefix(String prefix) {
        return dao.findByPrefix(prefix);
    }

    /**
     * Vérifie si un type de produit correspondant à un nom existe.
     *
     * @param name Le nom à vérifier
     * @return Si un type de produit ayant le nom existe
     */
    public boolean existsByName(String name) {
        return dao.existsByName(name);
    }

    /**
     * Vérifie si un type de produit correspondant à un préfixe existe.
     *
     * @param prefix Le préfixe à vérifier
     * @return Si un type de produit ayant le préfixe existe
     */
    public boolean existsByPrefix(String prefix) {
        return dao.existsByPrefix(prefix);
    }
}
