package dev.fyloz.trial.colorrecipesexplorer.core.services.model;

import dev.fyloz.trial.colorrecipesexplorer.core.exception.model.EntityNotFoundException;
import dev.fyloz.trial.colorrecipesexplorer.core.exception.model.ModelException;
import dev.fyloz.trial.colorrecipesexplorer.core.model.Material;
import dev.fyloz.trial.colorrecipesexplorer.core.model.MaterialType;
import dev.fyloz.trial.colorrecipesexplorer.core.model.MixType;
import dev.fyloz.trial.colorrecipesexplorer.core.services.GenericService;
import dev.fyloz.trial.colorrecipesexplorer.dao.MixTypeDao;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Optional;

@Service
public class MixTypeService extends GenericService<MixType, MixTypeDao> {

    @Autowired
    public MixTypeService(MixTypeDao mixTypeDao) {
        super(mixTypeDao, MixType.class);
    }

    /**
     * Récupère le type de mélange correspondant à un nom.
     *
     * @param name Le nom du type de mélange
     * @return Le type de mélange correspondant au nom
     */
    public MixType getByName(String name) {
        Optional<MixType> found = dao.findByName(name);
        if (found.isEmpty()) throw new EntityNotFoundException(type, ModelException.IdentifierType.NAME, name);

        return found.get();
    }

    /**
     * Récupère le type de mélange correspondant à un produit.
     *
     * @param material Le produit du type de mélange
     * @return Le type de mélange correspondant au produit
     */
    public MixType getByMaterial(Material material) {
        Optional<MixType> found = dao.findByMaterial(material);
        if (found.isEmpty())
            throw new EntityNotFoundException(type, ModelException.IdentifierType.OTHER, MixType.IDENTIFIER_MATERIAL_NAME, material);

        return found.get();
    }

    /**
     * Crée un type de mélange.
     *
     * @param name         Le nom du type de mélange
     * @param materialType Le type de produit du type de mélange
     * @return Le type de mélange créé
     */
    public MixType createByName(String name, MaterialType materialType) {
        Material mixTypeMaterial = new Material(name, 0f, true, materialType);
        MixType mixType = new MixType(name, mixTypeMaterial);

        return save(mixType);
    }
}
