package dev.fyloz.trial.colorrecipesexplorer.web.controller.creators;

import dev.fyloz.trial.colorrecipesexplorer.core.exception.model.EntityAlreadyExistsException;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ModelResponseBuilder;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ResponseCode;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ResponseDataType;
import dev.fyloz.trial.colorrecipesexplorer.core.model.Company;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.CompanyService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.servlet.ModelAndView;

import javax.validation.Valid;

import static dev.fyloz.trial.colorrecipesexplorer.web.WebsitePaths.CREATOR_COMPANY;

@Controller
public class CompanyCreatorController {

    private CompanyService companyService;

    @Autowired
    public CompanyCreatorController(CompanyService companyService) {
        this.companyService = companyService;
    }

    @GetMapping(CREATOR_COMPANY)
    public ModelAndView showCreationPage(ModelAndView model, Company company) {
        return new ModelResponseBuilder(model)
                .withView(CREATOR_COMPANY)
                .addResponseData(ResponseDataType.COMPANY, company)
                .build();
    }

    @PostMapping(value = CREATOR_COMPANY, consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE)
    public ModelAndView createCompany(@ModelAttribute @Valid Company company) {
        ModelResponseBuilder modelResponseBuilder = new ModelResponseBuilder();

        try {
            company = companyService.save(company);

            modelResponseBuilder.addResponseData(ResponseDataType.COMPANY_NAME, company.getName());
        } catch (EntityAlreadyExistsException ex) {
            modelResponseBuilder.addResponseCode(ResponseCode.COMPANY_ALREADY_EXIST, company.getName());
        }

        return showCreationPage(modelResponseBuilder.build(), company);
    }
}
