package dev.fyloz.trial.colorrecipesexplorer.web.controller.editors;

import dev.fyloz.trial.colorrecipesexplorer.core.exception.model.EntityAlreadyExistsException;
import dev.fyloz.trial.colorrecipesexplorer.core.exception.model.EntityNotFoundException;
import dev.fyloz.trial.colorrecipesexplorer.core.exception.model.ModelException;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ModelResponseBuilder;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ResponseCode;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ResponseDataType;
import dev.fyloz.trial.colorrecipesexplorer.core.model.MaterialType;
import dev.fyloz.trial.colorrecipesexplorer.core.model.dto.MaterialTypeEditorDto;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.MaterialTypeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.servlet.ModelAndView;

import static dev.fyloz.trial.colorrecipesexplorer.web.WebsitePaths.*;

@Controller
public class MaterialTypeEditorController {

    private MaterialTypeService materialTypeService;

    @Autowired
    public MaterialTypeEditorController(MaterialTypeService materialTypeService) {
        this.materialTypeService = materialTypeService;
    }

    @GetMapping(EDITOR_MATERIAL_TYPE)
    public ModelAndView getPage(ModelAndView model) {
        return new ModelResponseBuilder(model)
                .withView(EDITOR_MATERIAL_TYPE)
                .addResponseData(ResponseDataType.MATERIAL_TYPES, materialTypeService.getAll())
                .build();
    }

    @GetMapping(EDITOR_MATERIAL_TYPE_SPECIFIC)
    public ModelAndView getEditPage(ModelAndView model, @PathVariable Long id, MaterialType materialType) {
        ModelResponseBuilder responseBuilder = new ModelResponseBuilder(model).withView(EDITOR_MATERIAL_TYPE_EDITOR);

        try {
            if (materialType == null) materialType = materialTypeService.getById(id);

            return responseBuilder
                    .addResponseData(ResponseDataType.MATERIAL_TYPE, materialType)
                    .build();
        } catch (EntityNotFoundException ex) {
            return getPage(responseBuilder.addResponseCode(ResponseCode.MATERIAL_TYPE_NOT_FOUND, id).build());
        }
    }

    @PostMapping(value = EDITOR_MATERIAL_TYPE, consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE)
    public ModelAndView updateMaterialType(MaterialTypeEditorDto materialTypeDto) {
        ModelResponseBuilder responseBuilder = new ModelResponseBuilder();

        try {
            materialTypeService.update(materialTypeDto);

            return getPage(responseBuilder.addResponseCode(ResponseCode.SUCCESS_SAVING_MATERIAL_TYPE, materialTypeDto.getMaterialType().getName()).build());
        } catch (EntityNotFoundException ex) {
            return getPage(responseBuilder.addResponseCode(ResponseCode.MATERIAL_TYPE_NOT_FOUND, materialTypeDto.getOldName()).build());
        } catch (EntityAlreadyExistsException ex) {
            if (ModelException.IdentifierType.NAME.equals(ex.getIdentifierType()))
                responseBuilder.addResponseCode(ResponseCode.MATERIAL_TYPE_ALREADY_EXIST, materialTypeDto.getMaterialType().getName());
            else if (ex.getIdentifierName().equals(MaterialType.IDENTIFIER_PREFIX_NAME))
                responseBuilder.addResponseCode(ResponseCode.MATERIAL_TYPE_ALREADY_EXIST_PREFIX, materialTypeDto.getMaterialType().getPrefix());
        }

        return getEditPage(responseBuilder.build(), materialTypeDto.getMaterialType().getId(), materialTypeDto.getMaterialType());
    }
}
