package dev.fyloz.trial.colorrecipesexplorer.web.controller.editors;

import dev.fyloz.trial.colorrecipesexplorer.core.exception.model.EntityNotFoundException;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ModelResponseBuilder;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ResponseCode;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ResponseDataType;
import dev.fyloz.trial.colorrecipesexplorer.core.model.Mix;
import dev.fyloz.trial.colorrecipesexplorer.core.model.dto.MixFormDto;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.MaterialService;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.MaterialTypeService;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.MixService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;

import javax.validation.Valid;

import static dev.fyloz.trial.colorrecipesexplorer.web.WebsitePaths.*;

@Controller
public class MixEditorController {

    private MixService mixService;
    private MaterialService materialService;
    private MaterialTypeService materialTypeService;

    @Autowired
    public MixEditorController(MixService mixService, MaterialService materialService, MaterialTypeService materialTypeService) {
        this.mixService = mixService;
        this.materialService = materialService;
        this.materialTypeService = materialTypeService;
    }

    @GetMapping(EDITOR_MIX_SPECIFIC)
    public ModelAndView getPage(ModelAndView model, @PathVariable Long id) {
        ModelResponseBuilder modelResponseBuilder = new ModelResponseBuilder(model).withView(EDITOR_MIX);

        try {
            Mix mix = mixService.getById(id);

            return modelResponseBuilder
                    .addResponseData(ResponseDataType.MIX, mix)
                    .addResponseData(ResponseDataType.MATERIAL_TYPE, mix.getMixType().getMaterial().getMaterialType())
                    .addResponseData(ResponseDataType.MATERIAL_TYPES, materialTypeService.getAll())
                    .addResponseData(ResponseDataType.MIX_JSON, mixService.asJson(mix))
                    .addResponseData(ResponseDataType.MATERIALS_JSON, materialService.asJson(mixService.getAvailableMaterialsForMix(mix)))
                    .build();

        } catch (EntityNotFoundException ex) {
            return modelResponseBuilder
                    .withRedirect(EDITOR_RECIPE)
                    .build();
        }
    }

    @PostMapping(value = EDITOR_MIX, consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE)
    public ModelAndView updateMix(@ModelAttribute @Valid MixFormDto formDto, @RequestParam("mixId") Long id) {
        ModelResponseBuilder modelResponseBuilder = new ModelResponseBuilder().withRedirect(EDITOR_RECIPE_SPECIFIC, formDto.getRecipe().getId());

        try {
            Mix mix = mixService.getById(id);

            mixService.edit(mix, formDto);
        } catch (EntityNotFoundException ex) {
            return getPage(modelResponseBuilder.addResponseCode(ResponseCode.MIX_NOT_FOUND, id).build(), id);
        }

        return modelResponseBuilder.build();
    }

    @GetMapping(REMOVER_MIX_SPECIFIC)
    public ModelAndView deleteMix(@PathVariable Long id) {
        ModelResponseBuilder modelResponseBuilder = new ModelResponseBuilder();

        try {
            Mix mix = mixService.getById(id);
            mixService.delete(mix);

            return modelResponseBuilder
                    .withRedirect(EDITOR_RECIPE_SPECIFIC, mix.getRecipe().getId())
                    .build();
        } catch (EntityNotFoundException ex) {
            modelResponseBuilder.addResponseCode(ResponseCode.MIX_NOT_FOUND, id);
        }

        return getPage(modelResponseBuilder.build(), id);
    }
}
