package dev.fyloz.trial.colorrecipesexplorer.web.controller.removers;

import dev.fyloz.trial.colorrecipesexplorer.core.exception.model.EntityNotFoundException;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ModelResponseBuilder;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ResponseCode;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ResponseDataType;
import dev.fyloz.trial.colorrecipesexplorer.core.model.Recipe;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.RecipeService;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.servlet.ModelAndView;

import java.util.Optional;

import static dev.fyloz.trial.colorrecipesexplorer.web.WebsitePaths.REMOVER_RECIPE;
import static dev.fyloz.trial.colorrecipesexplorer.web.WebsitePaths.REMOVER_RECIPE_SPECIFIC;

@Controller
public class RecipeRemoverController {

    private RecipeService recipeService;

    public RecipeRemoverController(RecipeService recipeService) {
        this.recipeService = recipeService;
    }

    @GetMapping(REMOVER_RECIPE)
    public ModelAndView getPage(ModelAndView model) {
        return new ModelResponseBuilder(model)
                .withView(REMOVER_RECIPE)
                .addResponseData(ResponseDataType.RECIPE_MAP, recipeService.getRecipesByCompany())
                .build();
    }

    @PostMapping(REMOVER_RECIPE_SPECIFIC)
    public ModelAndView removeRecipe(@PathVariable Long id) {
        ModelResponseBuilder modelResponseBuilder = new ModelResponseBuilder();

        try {
            recipeService.deleteById(id);

            modelResponseBuilder.addResponseCode(ResponseCode.SUCCESS_DELETING_RECIPE, recipeService.getById(id).getName());
        } catch (EntityNotFoundException ex) {
            modelResponseBuilder.addResponseCode(ResponseCode.RECIPE_NOT_FOUND, id);
        }

        return getPage(modelResponseBuilder.build());
    }
}
